# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import json
import click
import pychrome


click.disable_unicode_literals_warning = True
CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


shared_options = [
    click.option("--host", "-t", type=click.STRING, default='127.0.0.1', help="HTTP frontend host"),
    click.option("--port", "-p", type=click.INT, default=9222, help="HTTP frontend port"),
    click.option("--secure", "-s", is_flag=True, help="HTTPS/WSS frontend")
]


def add_shared_options(func):
    for option in shared_options:
        func = option(func)

    return func


class JSONTabEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, pychrome.Tab):
            return obj._kwargs

        return super(JSONTabEncoder, self).default(self, obj)


@click.group(context_settings=CONTEXT_SETTINGS)
@click.version_option(pychrome.__version__)
def main():
    pass


@main.command(context_settings=CONTEXT_SETTINGS)
@add_shared_options
def list(host, port, secure):
    """list all the available targets/tabs"""
    url = "%s://%s:%s" % ("https" if secure else "http", host, port)
    try:
        browser = pychrome.Browser(url)
        click.echo(json.dumps(browser.list_tab(), cls=JSONTabEncoder, indent=4))
    except Exception as e:
        click.echo(e)


@main.command(context_settings=CONTEXT_SETTINGS)
@click.argument("url", required=False)
@add_shared_options
def new(host, port, secure, url="about:blank"):
    """create a new target/tab"""
    _url = "%s://%s:%s" % ("https" if secure else "http", host, port)

    try:
        browser = pychrome.Browser(_url)
        click.echo(json.dumps(browser.new_tab(url), cls=JSONTabEncoder, indent=4))
    except Exception as e:
        click.echo(e)


@main.command(context_settings=CONTEXT_SETTINGS)
@click.argument("id")
@add_shared_options
def activate(host, port, secure, id):
    """activate a target/tab by id"""
    url = "%s://%s:%s" % ("https" if secure else "http", host, port)

    try:
        browser = pychrome.Browser(url)
        click.echo(browser.activate_tab(id))
    except Exception as e:
        click.echo(e)


@main.command(context_settings=CONTEXT_SETTINGS)
@click.argument("id")
@add_shared_options
def close(host, port, secure, id):
    """close a target/tab by id"""
    url = "%s://%s:%s" % ("https" if secure else "http", host, port)

    try:
        browser = pychrome.Browser(url)
        click.echo(browser.close_tab(id))
    except Exception as e:
        click.echo(e)


@main.command(context_settings=CONTEXT_SETTINGS)
@add_shared_options
def version(host, port, secure):
    """show the browser version"""
    url = "%s://%s:%s" % ("https" if secure else "http", host, port)

    try:
        browser = pychrome.Browser(url)
        click.echo(json.dumps(browser.version(), indent=4))
    except Exception as e:
        click.echo(e)


if __name__ == '__main__':
    main()



import pychrome

browser = pychrome.Browser(url="http://127.0.0.1:9222")
tab = browser.new_tab()

def request_will_be_sent(**kwargs):
    print("loading: %s" % kwargs.get('request').get('url'))


tab.set_listener("Network.requestWillBeSent", request_will_be_sent)

tab.start()
tab.call_method("Network.enable")
tab.call_method("Page.navigate", url="https://github.com/fate0/pychrome", _timeout=5)

tab.wait(5)
tab.stop()

browser.close_tab(tab)



# -*- coding: utf-8 -*-

import time
import logging
import pychrome
import functools


logging.basicConfig(level=logging.INFO)
logging.getLogger("urllib3").setLevel(logging.WARNING)


def close_all_tabs(browser):
    if len(browser.list_tab()) == 0:
        return

    logging.debug("[*] recycle")
    for tab in browser.list_tab():
        browser.close_tab(tab)

    time.sleep(1)
    assert len(browser.list_tab()) == 0


def setup_function(function):
    browser = pychrome.Browser()
    close_all_tabs(browser)


def teardown_function(function):
    browser = pychrome.Browser()
    close_all_tabs(browser)


def new_multi_tabs(browser, n):
    tabs = []
    for i in range(n):
        tabs.append(browser.new_tab())

    return tabs


def test_normal_callmethod():
    browser = pychrome.Browser()
    tabs = new_multi_tabs(browser, 10)

    for tab in tabs:
        tab.start()
        result = tab.Page.navigate(url="http://www.fatezero.org")
        assert result['frameId']

    time.sleep(3)

    for tab in tabs:
        result = tab.Runtime.evaluate(expression="document.domain")
        assert result['result']['type'] == 'string'
        assert result['result']['value'] == 'www.fatezero.org'
        tab.stop()


def test_set_event_listener():
    browser = pychrome.Browser()
    tabs = new_multi_tabs(browser, 10)

    def request_will_be_sent(tab, **kwargs):
        tab.stop()

    for tab in tabs:
        tab.start()
        tab.Network.requestWillBeSent = functools.partial(request_will_be_sent, tab)
        tab.Network.enable()
        try:
            tab.Page.navigate(url="chrome://newtab/")
        except pychrome.UserAbortException:
            pass

    for tab in tabs:
        if not tab.wait(timeout=5):
            assert False, "never get here"
        tab.stop()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import requests

from .tab import Tab


__all__ = ["Browser"]


class Browser(object):
    _all_tabs = {}

    def __init__(self, url="http://127.0.0.1:9222"):
        self.dev_url = url

        if self.dev_url not in self._all_tabs:
            self._tabs = self._all_tabs[self.dev_url] = {}
        else:
            self._tabs = self._all_tabs[self.dev_url]

    def new_tab(self, url=None, timeout=None):
        url = url or ''
        rp = requests.get("%s/json/new?%s" % (self.dev_url, url), json=True, timeout=timeout)
        tab = Tab(**rp.json())
        self._tabs[tab.id] = tab
        return tab

    def __list_tab_by_type(self, type, timeout=None):
        rp = requests.get("%s/json" % self.dev_url, json=True, timeout=timeout)
        tabs_map = {}
        for tab_json in rp.json():
            if tab_json['type'] != 'page':  # pragma: no cover
                continue

            if tab_json['id'] in self._tabs and self._tabs[tab_json['id']].status != Tab.status_stopped:
                tabs_map[tab_json['id']] = self._tabs[tab_json['id']]
            else:
                tabs_map[tab_json['id']] = Tab(**tab_json)

        self._tabs = tabs_map
        return list(self._tabs.values())
    

    def list_tab(self, timeout=None):
        return self.__list_tab_by_type('page', timeout)
    
    def list_tab_backgroud(self, timeout=None):
        return self.__list_tab_by_type('background_page', timeout)

    def activate_tab(self, tab_id, timeout=None):
        if isinstance(tab_id, Tab):
            tab_id = tab_id.id

        rp = requests.get("%s/json/activate/%s" % (self.dev_url, tab_id), timeout=timeout)
        return rp.text

    def close_tab(self, tab_id, timeout=None):
        if isinstance(tab_id, Tab):
            tab_id = tab_id.id

        tab = self._tabs.pop(tab_id, None)
        if tab and tab.status == Tab.status_started:  # pragma: no cover
            tab.stop()

        rp = requests.get("%s/json/close/%s" % (self.dev_url, tab_id), timeout=timeout)
        return rp.text

    def version(self, timeout=None):
        rp = requests.get("%s/json/version" % self.dev_url, json=True, timeout=timeout)
        return rp.json()

    def __str__(self):
        return '<Browser %s>' % self.dev_url

    __repr__ = __str__



import pychrome

# create a browser instance
browser = pychrome.Browser(url="http://127.0.0.1:9222")

# create a tab
tab = browser.new_tab()

# register callback if you want
def request_will_be_sent(**kwargs):
    print("loading: %s" % kwargs.get('request').get('url'))

tab.Network.requestWillBeSent = request_will_be_sent

# start the tab
tab.start()

# call method
tab.Network.enable()
# call method with timeout
tab.Page.navigate(url="https://github.com/fate0/pychrome", _timeout=5)

# wait for loading
tab.wait(5)

# stop the tab (stop handle events and stop recv message from chrome)
tab.stop()

# close tab
browser.close_tab(tab)



# -*- coding: utf-8 -*-

import time
import logging
import pychrome


logging.basicConfig(level=logging.INFO)


def close_all_tabs(browser):
    if len(browser.list_tab()) == 0:
        return

    for tab in browser.list_tab():
        browser.close_tab(tab)

    time.sleep(1)
    assert len(browser.list_tab()) == 0


def setup_function(function):
    browser = pychrome.Browser()
    close_all_tabs(browser)


def teardown_function(function):
    browser = pychrome.Browser()
    close_all_tabs(browser)


def test_chome_version():
    browser = pychrome.Browser()
    browser_version = browser.version()
    assert isinstance(browser_version, dict)


def test_browser_list():
    browser = pychrome.Browser()
    tabs = browser.list_tab()
    assert len(tabs) == 0


def test_browser_new():
    browser = pychrome.Browser()
    browser.new_tab()
    tabs = browser.list_tab()
    assert len(tabs) == 1


def test_browser_activate_tab():
    browser = pychrome.Browser()
    tabs = []
    for i in range(10):
        tabs.append(browser.new_tab())

    for tab in tabs:
        browser.activate_tab(tab)


def test_browser_tabs_map():
    browser = pychrome.Browser()

    tab = browser.new_tab()
    assert tab in browser.list_tab()
    assert tab in browser.list_tab()

    browser.close_tab(tab)
    assert tab not in browser.list_tab()


def test_browser_new_10_tabs():
    browser = pychrome.Browser()
    tabs = []
    for i in range(10):
        tabs.append(browser.new_tab())

    time.sleep(1)
    assert len(browser.list_tab()) == 10

    for tab in tabs:
        browser.close_tab(tab.id)

    time.sleep(1)
    assert len(browser.list_tab()) == 0


def test_browser_new_100_tabs():
    browser = pychrome.Browser()
    tabs = []
    for i in range(100):
        tabs.append(browser.new_tab())

    time.sleep(1)
    assert len(browser.list_tab()) == 100

    for tab in tabs:
        browser.close_tab(tab)

    time.sleep(1)
    assert len(browser.list_tab()) == 0



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import time
import base64
import pychrome
import threading


urls = [
    "http://fatezero.org",
    "http://blog.fatezero.org",
    "http://github.com/fate0",
    "http://github.com/fate0/pychrome"
]


class EventHandler(object):
    screen_lock = threading.Lock()

    def __init__(self, browser, tab):
        self.browser = browser
        self.tab = tab
        self.start_frame = None

    def frame_started_loading(self, frameId):
        if not self.start_frame:
            self.start_frame = frameId

    def frame_stopped_loading(self, frameId):
        if self.start_frame == frameId:
            self.tab.Page.stopLoading()

            with self.screen_lock:
                # must activate current tab
                print(self.browser.activate_tab(self.tab.id))

                try:
                    data = self.tab.Page.captureScreenshot()
                    with open("%s.png" % time.time(), "wb") as fd:
                        fd.write(base64.b64decode(data['data']))
                finally:
                    self.tab.stop()


def close_all_tabs(browser):
    if len(browser.list_tab()) == 0:
        return

    for tab in browser.list_tab():
        try:
            tab.stop()
        except pychrome.RuntimeException:
            pass

        browser.close_tab(tab)

    time.sleep(1)
    assert len(browser.list_tab()) == 0


def main():
    browser = pychrome.Browser()

    close_all_tabs(browser)

    tabs = []
    for i in range(len(urls)):
        tabs.append(browser.new_tab())

    for i, tab in enumerate(tabs):
        eh = EventHandler(browser, tab)
        tab.Page.frameStartedLoading = eh.frame_started_loading
        tab.Page.frameStoppedLoading = eh.frame_stopped_loading

        tab.start()
        tab.Page.stopLoading()
        tab.Page.enable()
        tab.Page.navigate(url=urls[i])

    for tab in tabs:
        tab.wait(60)
        tab.stop()
        browser.close_tab(tab)

    print('Done')


if __name__ == '__main__':
    main()



import pychrome
import time
import urllib



class ChromiumClient(object):
    """ Client to interact with Chromium """

    def __init__(self):
        self.browser = pychrome.Browser(url="http://127.0.0.1:9222")

    def do_post(self):
        self.tab = self.browser.new_tab()

        event_handler = EventHandler()

        event_handler.set_token('asdkflj497564dsklf')
        event_handler.set_post_data({
            'param1': 'value1',
            'param2': 'value2'
        })

        url_pattern_object = {'urlPattern': '*fate0*'}
        self.tab.Network.setRequestInterception(patterns=[url_pattern_object])

        self.tab.Network.requestIntercepted = event_handler.on_request_intercepted

        self.tab.start()

        self.tab.Network.enable()
        self.tab.Page.enable()

        self.tab.Page.navigate(url='https://github.com/fate0/pychrome')

        self.tab.wait(5)
        self.tab.stop()

        self.browser.close_tab(self.tab.id)


class EventHandler(object):
    def __init__(self):
        self.tab = None
        self.token = None
        self.is_first_request = False
        self.post_data = {}

    def set_tab(self, t):
        self.tab = t

    def set_token(self, t):
        self.token = t

    def set_post_data(self, pd):
        self.post_data = pd

    def on_request_intercepted(self, **kwargs):
        new_args = {'interceptionId': kwargs['interceptionId']}

        if self.is_first_request:
            # Modify first request only, following are media/static
            # requests...
            self.is_first_request = False

            extra_headers = {
                'Requested-by': 'Chromium',
                'Authorization': 'Token ' + self.token
            }

            request = kwargs.get('request')
            request['headers'].update(extra_headers)

            new_args.update({
                'url': request['url'],
                'method': 'POST',
                'headers': request['headers'],
                'postData': urllib.urlencode(self.post_data)
            })

        self.tab.Network.continueInterceptedRequest(**new_args)


if __name__ == '__main__':
    client = ChromiumClient()

    client.do_post()


#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
require chrome version >= 61.0.3119.0
headless mode
"""


import time
import pychrome


class EventHandler(object):
    def __init__(self, browser, tab):
        self.browser = browser
        self.tab = tab
        self.start_frame = None
        self.is_first_request = True
        self.html_content = None

    def frame_started_loading(self, frameId):
        if not self.start_frame:
            self.start_frame = frameId

    def request_intercepted(self, interceptionId, request, **kwargs):
        if self.is_first_request:
            self.is_first_request = False
            headers = request.get('headers', {})
            headers['Test-key'] = 'test-value'
            self.tab.Network.continueInterceptedRequest(
                interceptionId=interceptionId,
                headers=headers,
                method='POST',
                postData="hello post data: %s" % time.time()
            )
        else:
            self.tab.Network.continueInterceptedRequest(
                interceptionId=interceptionId
            )

    def frame_stopped_loading(self, frameId):
        if self.start_frame == frameId:
            self.tab.Page.stopLoading()
            result = self.tab.Runtime.evaluate(expression="document.documentElement.outerHTML")
            self.html_content = result.get('result', {}).get('value', "")
            print(self.html_content)
            self.tab.stop()


def close_all_tabs(browser):
    if len(browser.list_tab()) == 0:
        return

    for tab in browser.list_tab():
        try:
            tab.stop()
        except pychrome.RuntimeException:
            pass

        browser.close_tab(tab)

    time.sleep(1)
    assert len(browser.list_tab()) == 0


def main():
    browser = pychrome.Browser()

    close_all_tabs(browser)

    tabs = []
    for i in range(4):
        tabs.append(browser.new_tab())

    for i, tab in enumerate(tabs):
        eh = EventHandler(browser, tab)
        tab.Network.requestIntercepted = eh.request_intercepted
        tab.Page.frameStartedLoading = eh.frame_started_loading
        tab.Page.frameStoppedLoading = eh.frame_stopped_loading

        tab.start()
        tab.Page.stopLoading()
        tab.Page.enable()
        tab.Network.setRequestInterceptionEnabled(enabled=True)
        tab.Page.navigate(url="http://httpbin.org/post")

    for tab in tabs:
        tab.wait(60)
        tab.stop()
        browser.close_tab(tab)

    print('Done')


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

import time
import logging
import pychrome

logging.basicConfig(level=logging.INFO)


def close_all_tabs(browser):
    if len(browser.list_tab()) == 0:
        return

    logging.debug("[*] recycle")
    for tab in browser.list_tab():
        browser.close_tab(tab)

    time.sleep(1)
    assert len(browser.list_tab()) == 0


def setup_function(function):
    browser = pychrome.Browser()
    close_all_tabs(browser)


def teardown_function(function):
    browser = pychrome.Browser()
    close_all_tabs(browser)


def test_normal_callmethod():
    browser = pychrome.Browser()
    tab = browser.new_tab()

    tab.start()
    result = tab.Page.navigate(url="http://www.fatezero.org")
    assert result['frameId']

    time.sleep(1)
    result = tab.Runtime.evaluate(expression="document.domain")

    assert result['result']['type'] == 'string'
    assert result['result']['value'] == 'www.fatezero.org'
    tab.stop()


def test_invalid_method():
    browser = pychrome.Browser()
    tab = browser.new_tab()

    tab.start()
    try:
        tab.Page.NotExistMethod()
        assert False, "never get here"
    except pychrome.CallMethodException:
        pass
    tab.stop()


def test_invalid_params():
    browser = pychrome.Browser()
    tab = browser.new_tab()

    tab.start()
    try:
        tab.Page.navigate()
        assert False, "never get here"
    except pychrome.CallMethodException:
        pass

    try:
        tab.Page.navigate("http://www.fatezero.org")
        assert False, "never get here"
    except pychrome.CallMethodException:
        pass

    try:
        tab.Page.navigate(invalid_params="http://www.fatezero.org")
        assert False, "never get here"
    except pychrome.CallMethodException:
        pass

    try:
        tab.Page.navigate(url="http://www.fatezero.org", invalid_params=123)
    except pychrome.CallMethodException:
        assert False, "never get here"

    tab.stop()


def test_set_event_listener():
    browser = pychrome.Browser()
    tab = browser.new_tab()

    def request_will_be_sent(**kwargs):
        tab.stop()

    tab.start()
    tab.Network.requestWillBeSent = request_will_be_sent
    tab.Network.enable()

    try:
        tab.Page.navigate(url="chrome://newtab/")
    except pychrome.UserAbortException:
        pass

    if not tab.wait(timeout=5):
        assert False, "never get here"


def test_set_wrong_listener():
    browser = pychrome.Browser()
    tab = browser.new_tab()

    tab.start()
    try:
        tab.Network.requestWillBeSent = "test"
        assert False, "never get here"
    except pychrome.RuntimeException:
        pass
    tab.stop()


def test_get_event_listener():
    browser = pychrome.Browser()
    tab = browser.new_tab()

    def request_will_be_sent(**kwargs):
        tab.stop()

    tab.start()
    tab.Network.requestWillBeSent = request_will_be_sent
    tab.Network.enable()
    try:
        tab.Page.navigate(url="chrome://newtab/")
    except pychrome.UserAbortException:
        pass

    if not tab.wait(timeout=5):
        assert False, "never get here"

    assert tab.Network.requestWillBeSent == request_will_be_sent
    tab.Network.requestWillBeSent = None

    assert not tab.get_listener("Network.requestWillBeSent")
    # notice this
    assert tab.Network.requestWillBeSent != tab.get_listener("Network.requestWillBeSent")

    tab.stop()


def test_reuse_tab_error():
    browser = pychrome.Browser()
    tab = browser.new_tab()

    def request_will_be_sent(**kwargs):
        tab.stop()

    tab.start()
    tab.Network.requestWillBeSent = request_will_be_sent
    tab.Network.enable()
    try:
        tab.Page.navigate(url="chrome://newtab/")
    except pychrome.UserAbortException:
        pass

    if not tab.wait(timeout=5):
        assert False, "never get here"

    try:
        tab.Page.navigate(url="http://www.fatezero.org")
        assert False, "never get here"
    except pychrome.RuntimeException:
        pass
    tab.stop()


def test_del_event_listener():
    browser = pychrome.Browser()
    tab = browser.new_tab()
    test_list = []

    def request_will_be_sent(**kwargs):
        test_list.append(1)
        tab.Network.requestWillBeSent = None

    tab.start()
    tab.Network.requestWillBeSent = request_will_be_sent
    tab.Network.enable()
    tab.Page.navigate(url="chrome://newtab/")
    tab.Page.navigate(url="http://www.fatezero.org")

    if tab.wait(timeout=5):
        assert False, "never get here"

    assert len(test_list) == 1
    tab.stop()


def test_del_all_event_listener():
    browser = pychrome.Browser()
    tab = browser.new_tab()
    test_list = []

    def request_will_be_sent(**kwargs):
        test_list.append(1)
        tab.del_all_listeners()

    tab.start()
    tab.Network.requestWillBeSent = request_will_be_sent
    tab.Network.enable()
    tab.Page.navigate(url="chrome://newtab/")

    if tab.wait(timeout=5):
        assert False, "never get here"

    assert len(test_list) == 1
    tab.stop()


class CallableClass(object):
    def __init__(self, tab):
        self.tab = tab

    def __call__(self, *args, **kwargs):
        self.tab.stop()


def test_use_callable_class_event_listener():
    browser = pychrome.Browser()
    tab = browser.new_tab()

    tab.start()
    tab.Network.requestWillBeSent = CallableClass(tab)
    tab.Network.enable()
    try:
        tab.Page.navigate(url="chrome://newtab/")
    except pychrome.UserAbortException:
        pass

    if not tab.wait(timeout=5):
        assert False, "never get here"

    tab.stop()


def test_status():
    browser = pychrome.Browser()
    tab = browser.new_tab()

    assert tab.status == pychrome.Tab.status_initial

    def request_will_be_sent(**kwargs):
        tab.stop()

    tab.Network.requestWillBeSent = request_will_be_sent

    assert tab.status == pychrome.Tab.status_initial

    tab.start()
    tab.Network.enable()
    assert tab.status == pychrome.Tab.status_started

    try:
        tab.Page.navigate(url="chrome://newtab/")
    except pychrome.UserAbortException:
        pass

    if not tab.wait(timeout=5):
        assert False, "never get here"

    tab.stop()
    assert tab.status == pychrome.Tab.status_stopped


def test_call_method_timeout():
    browser = pychrome.Browser()
    tab = browser.new_tab()

    tab.start()
    tab.Page.navigate(url="chrome://newtab/", _timeout=5)

    try:
        tab.Page.navigate(url="http://www.fatezero.org", _timeout=0.8)
    except pychrome.TimeoutException:
        pass

    try:
        tab.Page.navigate(url="http://www.fatezero.org", _timeout=0.005)
    except pychrome.TimeoutException:
        pass

    tab.stop()


def test_callback_exception():
    browser = pychrome.Browser()
    tab = browser.new_tab()

    def request_will_be_sent(**kwargs):
        raise Exception("test callback exception")

    tab.start()
    tab.Network.requestWillBeSent = request_will_be_sent
    tab.Network.enable()
    tab.Page.navigate(url="chrome://newtab/")

    if tab.wait(timeout=3):
        assert False, "never get here"

    tab.stop()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals


class PyChromeException(Exception):
    pass


class UserAbortException(PyChromeException):
    pass


class TabConnectionException(PyChromeException):
    pass


class CallMethodException(PyChromeException):
    pass


class TimeoutException(PyChromeException):
    pass


class RuntimeException(PyChromeException):
    pass
